#!/usr/bin/env python

# Copyright(c)2012-2013 Internet Archive. Software license AGPL version 3.
#
# This file is part of the `surt` python package.
#
#     surt is free software: you can redistribute it and/or modify
#     it under the terms of the GNU Affero General Public License as published by
#     the Free Software Foundation, either version 3 of the License, or
#     (at your option) any later version.
#
#     surt is distributed in the hope that it will be useful,
#     but WITHOUT ANY WARRANTY; without even the implied warranty of
#     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#     GNU Affero General Public License for more details.
#
#     You should have received a copy of the GNU Affero General Public License
#     along with surt.  If not, see <http://www.gnu.org/licenses/>.
#
#     The surt source is hosted at https://github.com/internetarchive/surt

"""This is a python port of URLRegexTransformer.java:
http://archive-access.svn.sourceforge.net/viewvc/archive-access/trunk/archive-access/projects/archive-commons/src/main/java/org/archive/url/URLRegexTransformer.java?view=markup
"""

import re

# stripPathSessionID
#_______________________________________________________________________________
_RES_PATH_SESSIONID = [
    re.compile(b"^(.*/)(\((?:[a-z]\([0-9a-z]{24}\))+\)/)([^\?]+\.aspx.*)$", re.I),
    re.compile(b"^(.*/)(\\([0-9a-z]{24}\\)/)([^\\?]+\\.aspx.*)$", re.I),
    ]

def stripPathSessionID(path):
    """It looks like the java version returns a lowercased path..
    So why does it uses a case-insensitive regex? We won't lowercase here.
    """
    for pattern in _RES_PATH_SESSIONID:
        m = pattern.match(path)
        if m:
            path = m.group(1) + m.group(3)

    return path


# stripQuerySessionID
#_______________________________________________________________________________
_RES_QUERY_SESSIONID = [
    re.compile(b"^(.*)(?:jsessionid=[0-9a-zA-Z]{32})(?:&(.*))?$", re.I),
    re.compile(b"^(.*)(?:phpsessid=[0-9a-zA-Z]{32})(?:&(.*))?$", re.I),
    re.compile(b"^(.*)(?:sid=[0-9a-zA-Z]{32})(?:&(.*))?$", re.I),
    re.compile(b"^(.*)(?:ASPSESSIONID[a-zA-Z]{8}=[a-zA-Z]{24})(?:&(.*))?$", re.I),
    re.compile(b"^(.*)(?:cfid=[^&]+&cftoken=[^&]+)(?:&(.*))?$", re.I),
    ]

def stripQuerySessionID(query):
    for pattern in _RES_QUERY_SESSIONID:
        m = pattern.match(query)
        if m:
            if m.group(2):
                query = m.group(1) + m.group(2)
            else:
                query = m.group(1)

    return query


# hostToSURT
#_______________________________________________________________________________
_RE_IP_ADDRESS = re.compile(br"(?:(?:\d{1,3}\.){3}\d{1,3})$")

def hostToSURT(host, reverse_ipaddr=True):
    if not reverse_ipaddr and _RE_IP_ADDRESS.match(host):
        return host

    parts = host.split(b'.')
    parts.reverse()
    return b','.join(parts)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# Copyright(c)2012-2013 Internet Archive. Software license AGPL version 3.
#
# This file is part of the `surt` python package.
#
#     surt is free software: you can redistribute it and/or modify
#     it under the terms of the GNU Affero General Public License as published by
#     the Free Software Foundation, either version 3 of the License, or
#     (at your option) any later version.
#
#     surt is distributed in the hope that it will be useful,
#     but WITHOUT ANY WARRANTY; without even the implied warranty of
#     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#     GNU Affero General Public License for more details.
#
#     You should have received a copy of the GNU Affero General Public License
#     along with surt.  If not, see <http://www.gnu.org/licenses/>.
#
#     The surt source is hosted at https://github.com/internetarchive/surt

from __future__ import absolute_import

import re
import tldextract
import collections

try:
    from urllib.parse import SplitResultBytes
except:
    from urlparse import SplitResult as SplitResultBytes

from surt.URLRegexTransformer import hostToSURT

_RE_MULTIPLE_PROTOCOLS = re.compile(br'^(https?://)+')
_RE_HAS_PROTOCOL = re.compile(b"^([a-zA-Z][a-zA-Z0-9\+\-\.]*):")
_RE_SPACES = re.compile(b'[\n\r\t]')

class handyurl(object):
    """A python port of the archive-commons org.archive.url HandyURL class

    To simplify the surt module, we add the URLParser.parse method here,
    which makes the URLParser class unnecessary. handyurl becomes a thin
    wrapper around python's urlparse module.

    Init an empty class:
    >>> h = handyurl()

    Init with just a host:
    >>> h = handyurl(host='www.amazon.co.uk')

    This version of handyurl contains a field for last_delimiter, to allow
    a url to roundtrip through this class without modification. From the
    urlparse docs:
    "This [roundtripping] may result in a slightly different, but equivalent URL,
    if the URL that was parsed originally had unnecessary delimiters
    (for example, a ? with an empty query; the RFC states that these are equivalent)."
    We want the url http://www.google.com/? to work, since there is a test for
    it in the GoogleURLCanonicalizer class. Note, however, the IAURLCanonicalizer
    class strips empty queries.
    """
    DEFAULT_PORT = None

    # init
    #___________________________________________________________________________
    def __init__(self, scheme=None, authUser=None, authPass=None,
                 host=None, port=DEFAULT_PORT, path=None,
                 query=None, hash=None, last_delimiter=None):
        self.scheme   = scheme
        self.authUser = authUser
        self.authPass = authPass
        self.host     = host
        self.port     = port
        self.path     = path
        self.query    = query
        self.hash     = hash
        self.last_delimiter = last_delimiter #added in python version


    '''
    RFC 2396-inspired regex.

    From the RFC Appendix B:
    <pre>
    URI Generic Syntax                August 1998

    B. Parsing a URI Reference with a Regular Expression

    As described in Section 4.3, the generic URI syntax is not sufficient
    to disambiguate the components of some forms of URI.  Since the
    "greedy algorithm" described in that section is identical to the
    disambiguation method used by POSIX regular expressions, it is
    natural and commonplace to use a regular expression for parsing the
    potential four components and fragment identifier of a URI reference.

    The following line is the regular expression for breaking-down a URI
    reference into its components.

    ^(([^:/?#]+):)?(//([^/?#]*))?([^?#]*)(\?([^#]*))?(#(.*))?
     12            3  4          5       6  7        8 9

    The numbers in the second line above are only to assist readability;
    they indicate the reference points for each subexpression (i.e., each
    paired parenthesis).  We refer to the value matched for subexpression
    <n> as $<n>.  For example, matching the above expression to

    http://www.ics.uci.edu/pub/ietf/uri/#Related

    results in the following subexpression matches:

    $1 = http:
    $2 = http
    $3 = //www.ics.uci.edu
    $4 = www.ics.uci.edu
    $5 = /pub/ietf/uri/
    $6 = <undefined>
    $7 = <undefined>
    $8 = #Related
    $9 = Related

    where <undefined> indicates that the component is not present, as is
    the case for the query component in the above example.  Therefore, we
    can determine the value of the four components and fragment as

    scheme    = $2
    authority = $4
    path      = $5
    query     = $7
    fragment  = $9
    </pre>

    --
    Below differs from the rfc regex in that...
    (1) we allow a URI made of a fragment only (Added extra
    group so indexing is off by one after scheme).
    (2) scheme is limited to legal scheme characters

    1: scheme:
    2: scheme
    3: //authority/path
    4: //authority
    5: authority (aka netloc)
    6: path
    7: ?query
    8: query
    9: #fragment
    A: fragment
    '''
    RFC2396REGEX = re.compile(br'^(([a-zA-Z][a-zA-Z0-9+.-]*):)?((//([^/?#]*))?([^?#]*)(\?([^#]*))?)?(#(.*))?$')
    # group open:                 12                           34  5          6       7  8          9 A
    # group close:                                         2 1             54        6         87 3      A9

    @classmethod
    def urlsplit(cls, url):
        """Similar to urllib.parse.urlsplit, but does not try to decode raw
        bytes. (Library method fails on non-ascii)"""
        assert isinstance(url, bytes)

        m = cls.RFC2396REGEX.match(url)
        assert m

        return SplitResultBytes(m.group(2) or b'', m.group(5) or b'',
                                m.group(6) or b'', m.group(8) or b'',
                                m.group(10) or b'')

    # parse() classmethod
    #___________________________________________________________________________
    @classmethod
    def parse(cls, url):
        u"""This method was in the java URLParser class, but we don't need
        a whole class to parse a url, when we can just use python's urlparse.

        """

        if not isinstance(url, bytes):
            url = url.encode('utf-8')

        # Note RE_SPACES does not match regular space (0x20). That is,
        # regular spaces are removed at head and tail, but not in the middle.
        # There's a test case for GoogleURLCanonicalizer.canonicalize that
        # asserts this behavior.
        url = url.strip()
        url = _RE_SPACES.sub(b'', url)

        url = cls.addDefaultSchemeIfNeeded(url)

        #From Tymm: deal with http://https/order.1and1.com
        url = _RE_MULTIPLE_PROTOCOLS.sub(lambda m: m.group(1), url)

        o = cls.urlsplit(url)

        scheme   = o.scheme   or None
        query    = o.query    or None
        fragment = o.fragment or None

        """Deal with hostnames that end with ':' without being followed by a port number"""
        if o.netloc.endswith(b':'):
            o = o._replace(netloc=o.netloc.rstrip(b':'))
        port     = o.port     or None

        hostname = o.hostname or None
        path     = o.path     or None

        if scheme.startswith(b'http'):
            #deal with "http:////////////////www.vikings.com"
            if hostname is None and path is not None:
                parts    = path.lstrip(b'/').partition(b'/')
                hostname = parts[0]
                path     = b'/'+parts[2]

        h = cls(scheme = scheme,
                host   = hostname,
                path   = path,
                query  = query,
                hash   = fragment,
                port   = port,
               )

        #See note at top about last_delimiter
        if url.endswith(b'?') and None == h.query:
            h.last_delimiter = b'?'

        return h

    # addDefaultSchemeIfNeeded()
    #___________________________________________________________________________
    """copied from URLParser.java"""
    @classmethod
    def addDefaultSchemeIfNeeded(cls, url):
        if not url:
            return url

        ###noah: accept anything that looks like it starts with a scheme:
        if _RE_HAS_PROTOCOL.match(url):
            return url
        else:
            return b"http://"+url

    # geturl()
    #___________________________________________________________________________
    def geturl(self):
        """urlparse.ParseResult has a geturl() method, so we have one too.
        Nicer than typing the java method name!
        """
        return self.getURLString()

    # getURLString()
    #___________________________________________________________________________
    def getURLString(self, **options):
        return self.geturl_bytes(**options).decode('utf-8')

    def geturl_bytes(self,
                     surt=False,
                     public_suffix=False,
                     trailing_comma=False,
                     reverse_ipaddr=True,
                     with_scheme=True,
                     **options):
        hostSrc = self.host
        if hostSrc:
            if public_suffix:
                hostSrc = self.getPublicSuffix()
            if surt:
                hostSrc = hostToSURT(hostSrc, reverse_ipaddr)

        if with_scheme:
            s = self.scheme + b':'
            if hostSrc:
                if self.scheme != b'dns':
                    s += b'//'
                if surt:
                    s += b"("
        elif not hostSrc:
            s = self.scheme + b':'
        else:
            s = b''

        if hostSrc:
            if self.authUser:
                s += self.authUser
                if self.authPass:
                    s += self.authPass
                s += b'@'

            s += hostSrc

            if self.port != self.DEFAULT_PORT:
                s += (":%d" % self.port).encode('utf-8')

            if surt:
                if trailing_comma:
                    s += b','
                s += b')'

        if self.path:
            s += self.path
        elif self.query is not None or self.hash is not None:
            #must have '/' with query or hash:
            s += b'/'

        if None != self.query:
            s += b'?' + self.query
        if None != self.hash:
            s += b'#' + self.hash

        if None != self.last_delimiter:
            s += self.last_delimiter

        return s

    # getPublicSuffix
    #___________________________________________________________________________
    def getPublicSuffix(self):
        """Uses the tldextract module to get the public suffix via the
        Public Suffix List.
        """
        return tldextract.extract(self.host).registered_domain.encode('ascii')

    # getPublicPrefix
    #___________________________________________________________________________
    def getPublicPrefix(self):
        """Uses the tldextract module to get the subdomain, using the
        Public Suffix List.
        """
        return tldextract.extract(self.host).subdomain

    # repr
    #___________________________________________________________________________
    # commented out because of http://bugs.python.org/issue5876
    # "__repr__ returning unicode doesn't work when called implicitly"
    #def __repr__(self):
    #    return u"""handyurl(scheme=%s, authUser=%s, authPass=%s, host=%s, port=%s, path=%s, query=%s, hash=%s)""".encode('utf-8') % (self.scheme, self.authUser, self.authPass, self.host, self.port, self.path, self.query, self.hash)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# Copyright(c)2012-2013 Internet Archive. Software license AGPL version 3.
#
# This file is part of the `surt` python package.
#
#     surt is free software: you can redistribute it and/or modify
#     it under the terms of the GNU Affero General Public License as published by
#     the Free Software Foundation, either version 3 of the License, or
#     (at your option) any later version.
#
#     surt is distributed in the hope that it will be useful,
#     but WITHOUT ANY WARRANTY; without even the implied warranty of
#     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#     GNU Affero General Public License for more details.
#
#     You should have received a copy of the GNU Affero General Public License
#     along with surt.  If not, see <http://www.gnu.org/licenses/>.
#
#     The surt source is hosted at https://github.com/internetarchive/surt

"""This is a python port of GoogleURLCanonicalizer.java:
http://archive-access.svn.sourceforge.net/viewvc/archive-access/trunk/archive-access/projects/archive-commons/src/main/java/org/archive/url/GoogleURLCanonicalizer.java?view=markup
"""

from __future__ import absolute_import

import re
import struct
import socket
import encodings.idna

from surt.handyurl import handyurl

try:
    from urllib.parse import quote_from_bytes, unquote_to_bytes
except:
    from urllib import quote as quote_from_bytes, unquote as unquote_to_bytes
from six import text_type, binary_type

# canonicalize()
#_______________________________________________________________________________
def canonicalize(url, **_ignored):
    url.hash = None
    if url.authUser:
        url.authUser = minimalEscape(url.authUser)
    if url.authPass:
        url.authPass = minimalEscape(url.authPass)
    if url.query:
        url.query = minimalEscape(url.query)

    if url.host:
        host = unescapeRepeatedly(url.host)
        try:
            host.decode('ascii')
        except UnicodeDecodeError:
            try:
                host = host.decode('utf-8', 'ignore').encode('idna')
            except ValueError:
                pass

        host = host.replace(b'..', b'.').strip(b'.')

        ip = attemptIPFormats(host)
        if ip:
            host = ip;
        else:
            host = escapeOnce(host.lower())

        url.host = host

    path = unescapeRepeatedly(url.path)
    if url.host:
        path = normalizePath(path)
    # else path is free-form sort of thing, not /directory/thing
    url.path = escapeOnce(path)

    return url

# normalizePath()
#_______________________________________________________________________________

def normalizePath(path):
    if not path:
        return b'/'

    #gives an empty trailing element if path ends with '/':
    paths       = path.split(b'/')
    keptPaths   = []
    first       = True

    for p in paths:
        if first:
            first = False
            continue
        elif b'.' == p:
            # skip
            continue
        elif b'..' == p:
            #pop the last path, if present:
            if len(keptPaths) > 0:
                keptPaths = keptPaths[:-1]
            else:
                # TODO: leave it? let's do for now...
                keptPaths.append(p)
        else:
            keptPaths.append(p)

    path = b'/'

    # If the path ends in '/', then the last element of keptPaths will be ''
    # Since we add a trailing '/' after the second-to-last element of keptPaths
    # in the for loop below, the trailing slash is preserved.
    numKept = len(keptPaths)
    if numKept > 0:
        for i in range(0, numKept-1):
            p = keptPaths[i]
            if len(p) > 0:
                #this will omit multiple slashes:
                path += p + b'/'
        path += keptPaths[numKept-1]

    return path

OCTAL_IP = re.compile(br"^(0[0-7]*)(\.[0-7]+)?(\.[0-7]+)?(\.[0-7]+)?$")
DECIMAL_IP = re.compile(br"^([1-9][0-9]*)(\.[0-9]+)?(\.[0-9]+)?(\.[0-9]+)?$")

# attemptIPFormats()
#_______________________________________________________________________________
def attemptIPFormats(host):
    if None == host:
        return None

    if host.isdigit():
        #mask hostname to lower four bytes to workaround issue with liveweb arc files
        return socket.inet_ntoa(
                struct.pack('>L', int(host) & 0xffffffff)).encode('ascii')
    else:
        m = DECIMAL_IP.match(host)
        if m:
            try:
                return socket.gethostbyname_ex(host)[2][0].encode('ascii')
            except (socket.gaierror, socket.herror):
                return None
        else:
            m = OCTAL_IP.match(host)
            if m:
                try:
                    return socket.gethostbyname_ex(host)[2][0].encode('ascii')
                except socket.gaierror:
                    return None

    return None


# minimalEscape()
#_______________________________________________________________________________
def minimalEscape(input):
    return escapeOnce(unescapeRepeatedly(input))

# escapeOnce()
#_______________________________________________________________________________
def escapeOnce(input):
    """escape everything outside of 32-128, except #"""
    if input:
        return quote_from_bytes(
                input, safe=b'''!"$&'()*+,-./:;<=>?@[\]^_`{|}~''').encode(
                        'ascii')
    else:
        return input


# unescapeRepeatedly()
#_______________________________________________________________________________
def unescapeRepeatedly(input):
    '''Argument may be str or bytes. Returns bytes.'''
    if None == input:
        return None

    while True:
        un = unquote_to_bytes(input)
        if un == input:
            return input
        input = un




#!/usr/bin/env python

# Copyright(c)2012-2013 Internet Archive. Software license AGPL version 3.
#
# This file is part of the `surt` python package.
#
#     surt is free software: you can redistribute it and/or modify
#     it under the terms of the GNU Affero General Public License as published by
#     the Free Software Foundation, either version 3 of the License, or
#     (at your option) any later version.
#
#     surt is distributed in the hope that it will be useful,
#     but WITHOUT ANY WARRANTY; without even the implied warranty of
#     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#     GNU Affero General Public License for more details.
#
#     You should have received a copy of the GNU Affero General Public License
#     along with surt.  If not, see <http://www.gnu.org/licenses/>.
#
#     The surt source is hosted at https://github.com/internetarchive/surt

"""A python port of the archive-commons org.archive.url HandyURL class

The original java version is here:
http://archive-access.svn.sourceforge.net/viewvc/archive-access/trunk/archive-access/projects/archive-commons/src/main/java/org/archive/url/
"""

from __future__ import absolute_import

from surt.handyurl import handyurl
from surt.surt import surt


__all__= [
    'handyurl',
    'surt'
]



#!/usr/bin/env python

# Copyright(c)2012-2013 Internet Archive. Software license AGPL version 3.
#
# This file is part of the `surt` python package.
#
#     surt is free software: you can redistribute it and/or modify
#     it under the terms of the GNU Affero General Public License as published by
#     the Free Software Foundation, either version 3 of the License, or
#     (at your option) any later version.
#
#     surt is distributed in the hope that it will be useful,
#     but WITHOUT ANY WARRANTY; without even the implied warranty of
#     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#     GNU Affero General Public License for more details.
#
#     You should have received a copy of the GNU Affero General Public License
#     along with surt.  If not, see <http://www.gnu.org/licenses/>.
#
#     The surt source is hosted at https://github.com/internetarchive/surt

"""This is a python port of the WaybackURLKeyMaker.java class:

http://archive-access.svn.sourceforge.net/viewvc/archive-access/trunk/archive-access/projects/archive-commons/src/main/java/org/archive/url/WaybackURLKeyMaker.java?view=markup
"""

from __future__ import absolute_import

from surt.handyurl import handyurl
from surt.URLRegexTransformer import hostToSURT

import surt.DefaultIAURLCanonicalizer as DefaultIAURLCanonicalizer

class CompositeCanonicalizer(object):
    def __init__(self, canonicalizers):
        self.canonicalizers = [
            self._normalize(canon) for canon in canonicalizers
            ]
    def __call__(self, hurl, **options):
        for canon in self.canonicalizers:
            hurl = canon(hurl, **options)
        return hurl
    @staticmethod
    def _normalize(canonicalizer):
        if hasattr(canonicalizer, '__call__'):
            return canonicalizer
        if hasattr(canonicalizer, 'canonicalize'):
            return canonicalizer.canonicalize
        raise AttributeError('canonicalizer must either be callable or have'
                             ' "canonicalizer" method')

# surt()
#_______________________________________________________________________________
def surt(url, canonicalizer=None, **options):
    if isinstance(url, bytes):
        return _surt_bytes(url, canonicalizer, **options)
    else:
        if url is not None:
            url = url.encode('utf-8')
        return _surt_bytes(url, canonicalizer, **options).decode('utf-8')

def _surt_bytes(url, canonicalizer, **options):
    if not url:
        return b"-"

    if url.startswith(b"filedesc"):
        return url

    if canonicalizer is None:
        canonicalizer = DefaultIAURLCanonicalizer.canonicalize
    else:
        if isinstance(canonicalizer, (list, tuple)):
            canonicalizer = CompositeCanonicalizer(canonicalizer)
        elif (not hasattr(canonicalizer, '__call__') and
              hasattr(canonicalizer, 'canonicalize')):
            canonicalizer = canonicalizer.canonicalize

    options.setdefault('surt', True)
    options.setdefault('with_scheme', False)

    hurl = canonicalizer(handyurl.parse(url), **options)
    return hurl.geturl_bytes(**options)



#!/usr/bin/env python

# Copyright(c)2012-2013 Internet Archive. Software license AGPL version 3.
#
# This file is part of the `surt` python package.
#
#     surt is free software: you can redistribute it and/or modify
#     it under the terms of the GNU Affero General Public License as published by
#     the Free Software Foundation, either version 3 of the License, or
#     (at your option) any later version.
#
#     surt is distributed in the hope that it will be useful,
#     but WITHOUT ANY WARRANTY; without even the implied warranty of
#     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#     GNU Affero General Public License for more details.
#
#     You should have received a copy of the GNU Affero General Public License
#     along with surt.  If not, see <http://www.gnu.org/licenses/>.
#
#     The surt source is hosted at https://github.com/internetarchive/surt

"""This is a python port of IAURLCanonicalizer.java:
http://archive-access.svn.sourceforge.net/viewvc/archive-access/trunk/archive-access/projects/archive-commons/src/main/java/org/archive/url/IAURLCanonicalizer.java?view=markup
"""

from __future__ import absolute_import

import re

from surt.handyurl import handyurl
from surt.URLRegexTransformer import stripPathSessionID, stripQuerySessionID

# canonicalize()
#_______________________________________________________________________________
def canonicalize(url, host_lowercase=True, host_massage=True,
                 auth_strip_user=True, auth_strip_pass=True,
                 port_strip_default=True, path_strip_empty=False,
                 path_lowercase=True, path_strip_session_id=True,
                 path_strip_trailing_slash_unless_empty=True,
                 query_lowercase=True, query_strip_session_id=True,
                 query_strip_empty=True, query_alpha_reorder=True,
                 hash_strip=True, **_ignored):
    """The input url is a handyurl instance"""
    if host_lowercase and url.host:
        url.host = url.host.lower()

    if host_massage and url.host and (url.scheme != b'dns'): ###java version calls massageHost regardless of scheme
        url.host = massageHost(url.host)

    if auth_strip_user:
        url.authUser = None
        url.authPass = None
    elif auth_strip_pass:
        url.arthPass = None

    if port_strip_default and url.scheme:
        defaultPort = getDefaultPort(url.scheme)
        if url.port == defaultPort:
            url.port = handyurl.DEFAULT_PORT

    path = url.path
    if path_strip_empty and b'/' == path:
        url.path = None
    else:
        if path_lowercase and path:
            path = path.lower()
        if path_strip_session_id and path:
            path = stripPathSessionID(path)
        if path_strip_empty and b'/' == path:
            path = None
        if path_strip_trailing_slash_unless_empty and path:
            if path.endswith(b'/') and len(path)>1:
                path = path[:-1]

        url.path = path

    query = url.query
    if query:
        if len(query) > 0:
            if query_strip_session_id:
                query = stripQuerySessionID(query)
            if query_lowercase:
                query = query.lower()
            if query_alpha_reorder:
                query = alphaReorderQuery(query)
        if b'' == query and query_strip_empty:
            query = None
        url.query = query
    else:
        if query_strip_empty:
            url.last_delimiter = None

    return url


# alphaReorderQuery()
#_______________________________________________________________________________
def alphaReorderQuery(orig):
    """It's a shame that we can't use urlparse.parse_qsl() for this, but this
    function does keeps the trailing '=' if there is a query arg with no value:
    "?foo" vs "?foo=", and we want to exactly match the java version
    """


    if None == orig:
        return None

    if len(orig) <= 1:
        return orig

    args = orig.split(b'&')
    qas = [tuple(arg.split(b'=', 1)) for arg in args]
    qas.sort()

    s = b''
    for t in qas:
        if 1 == len(t):
            s += t[0] + b'&'
        else:
            s += t[0] + b'=' + t[1] + b'&'

    return s[:-1] #remove last &


# massageHost()
#_______________________________________________________________________________
_RE_WWWDIGITS = re.compile(b'www\d*\.')

def massageHost(host):
    m = _RE_WWWDIGITS.match(host)
    if m:
        return host[len(m.group(0)):]
    else:
        return host

# getDefaultPort()
#_______________________________________________________________________________
def getDefaultPort(scheme):
    scheme_lower = scheme.lower()
    if b'http' == scheme_lower:
        return 80
    elif b'https' == scheme_lower:
        return 443
    else:
        return 0




from setuptools import setup
from setuptools.command.test import test as TestCommand

class PyTest(TestCommand):
    def finalize_options(self):
        TestCommand.finalize_options(self)
        self.test_suite = True

    def run_tests(self):
        import pytest
        import sys
        cmdline = ' -v --cov surt tests/'
        errcode = pytest.main(cmdline)
        sys.exit(errcode)


setup(name='surt',
      version='0.3.1',
      author='rajbot',
      author_email='raj@archive.org',
      classifiers=[
        'License :: OSI Approved :: GNU Affero General Public License v3',
      ],
      description='Sort-friendly URI Reordering Transform (SURT) python package.',
      long_description=open('README.rst').read(),
      url='https://github.com/internetarchive/surt',
      zip_safe=True,
      install_requires=[
          'six',
          'tldextract>=2.0',
      ],
      provides=[ 'surt' ],
      packages=[ 'surt' ],
      scripts=[],
      # Tests
      tests_require=[ 'pytest', 'pytest-cov' ],
      test_suite='',
      cmdclass={'test': PyTest},
     )
